<?php
/**
 * The admin-specific functionality of the plugin.
 *
 * @link       https://www.webtoffee.com
 * @since      1.0.0
 *
 * @package    Webtoffee_Product_Feed_Sync
 * @subpackage Webtoffee_Product_Feed_Sync/admin
 */
if ( !defined( 'ABSPATH' ) ) {
	exit;
}

/**
 * The admin-specific functionality of the plugin.
 *
 * Defines the plugin name, version, and two examples hooks for how to
 * enqueue the admin-specific stylesheet and JavaScript.
 *
 * @package    Webtoffee_Product_Feed_Sync
 * @subpackage Webtoffee_Product_Feed_Sync/admin
 * @author     WebToffee <info@webtoffee.com>
 */
class Webtoffee_Product_Feed_Sync_Admin {

	/**
	 * The ID of this plugin.
	 *
	 * @since    1.0.0
	 * @access   private
	 * @var      string    $plugin_name    The ID of this plugin.
	 */
	private $plugin_name;

	/**
	 * The version of this plugin.
	 *
	 * @since    1.0.0
	 * @access   private
	 * @var      string    $version    The current version of this plugin.
	 */
	private $version;

	/** @var string|null the generated external merchant settings ID */
	private $external_business_id;

	/** @var string the product sync to FB batch limit */
	public $batch_limit = 10;

	/** @var array the page handles that with the plugin views */
	public $wt_pages = array( 'woocommerce_page_webtoffee-product-feed' );

	/**
	 * Initialize the class and set its properties.
	 *
	 * @since    1.0.0
	 * @param      string    $plugin_name       The name of this plugin.
	 * @param      string    $version    The version of this plugin.
	 */
	public function __construct( $plugin_name, $version ) {

		$this->plugin_name	 = $plugin_name;
		$this->version		 = $version;
	}

	/**
	 * Register the stylesheets for the admin area.
	 *
	 * @since    1.0.0
	 */
	public function enqueue_styles() {

		/**
		 * This function is provided for demonstration purposes only.
		 *
		 * An instance of this class should be passed to the run() function
		 * defined in Webtoffee_Product_Feed_Sync_Loader as all of the hooks are defined
		 * in that particular class.
		 *
		 * The Webtoffee_Product_Feed_Sync_Loader will then create the relationship
		 * between the defined hooks and the functions defined in this
		 * class.
		 */
		$current_screen = get_current_screen();

		if ( isset( $current_screen->id ) && in_array( $current_screen->id, $this->wt_pages ) ) {

			wp_enqueue_style( $this->plugin_name, plugin_dir_url( __FILE__ ) . 'css/webtoffee-product-feed-admin.css', array(), $this->version, 'all' );
		}
	}

	/**
	 * Register the JavaScript for the admin area.
	 *
	 * @since    1.0.0
	 */
	public function enqueue_scripts() {

		/**
		 * This function is provided for demonstration purposes only.
		 *
		 * An instance of this class should be passed to the run() function
		 * defined in Webtoffee_Product_Feed_Sync_Loader as all of the hooks are defined
		 * in that particular class.
		 *
		 * The Webtoffee_Product_Feed_Sync_Loader will then create the relationship
		 * between the defined hooks and the functions defined in this
		 * class.
		 */
		$current_screen = get_current_screen();

		if ( isset( $current_screen->id ) && in_array( $current_screen->id, $this->wt_pages ) ) {

			wp_enqueue_script( $this->plugin_name, plugin_dir_url( __FILE__ ) . 'js/webtoffee-product-feed-admin.js', array( 'jquery' ), $this->version, false );
			wp_enqueue_script( $this->plugin_name . '-steps', plugin_dir_url( __FILE__ ) . 'js/jquery.steps.js', array( 'jquery' ), $this->version, false );
			$params=array(
			'nonces' => array(
		        'main' => wp_create_nonce(WEBTOFFEE_PRODUCT_FEED_ID),
		     ),
			'ajax_url' => admin_url('admin-ajax.php'),
			'plugin_id' =>WEBTOFFEE_PRODUCT_FEED_ID,
			'msgs'=>array(
				
				'error'=>__('Error.'),
				'success'=>__('Success.'),
				'loading'=>__('Loading...'),
				'process' => __('Processing Sync...'),

			)
		);
		wp_localize_script($this->plugin_name, 'wt_feed_params', $params);
		}
	}

	/**
	 * Adds the Facebook menu item.
	 *
	 * @since 2.0.0
	 */
	public function add_menu_item() {

		add_submenu_page( 'woocommerce', __( 'Facebook Catalog Manager', 'webtoffee-product-feed' ), __( 'Facebook Catalog', 'webtoffee-product-feed' ), 'manage_woocommerce', WT_Fb_Catalog_Manager_Settings::PAGE_ID, [ $this, 'render' ], 5 );
		add_submenu_page( 'woocommerce', __( 'Facebook Category Mapping', 'webtoffee-product-feed' ), __( 'Facebook Category Mapping', 'webtoffee-product-feed' ), 'manage_woocommerce', 'wt-fbfeed-category-mapping', [ $this, 'wt_fbfeed_category_mapping' ], 6 );
		//add_submenu_page( 'woocommerce', __( 'Facebook Attribute Mapping', 'webtoffee-product-feed' ), __( 'Facebook Attribute Mapping', 'webtoffee-product-feed' ), 'manage_woocommerce', 'wt-fbfeed-attribute-mapping', [ $this, 'wt_fbfeed_attribute_mapping' ], 7 );
	}

	
	
	/**
	 * Show action links on the plugin screen.
	 *
	 * @param mixed $links Plugin action links.
	 *
	 * @return array
	 */
	public function add_productfeed_action_links( $links ) {

	$plugin_links = array(
		'<a href="' . esc_url( admin_url( 'admin.php?page=webtoffee-product-feed' ) ) . '">' . __( 'Settings', 'webtoffee-product-feed' ) . '</a>',
		'<a target="_blank" href="https://wordpress.org/support/plugin/webtoffee-product-feed/">' . __( 'Support', 'webtoffee-product-feed' ) . '</a>',
		'<a target="_blank" href="https://www.webtoffee.com/webtoffee-product-feed-user-guide/">' . __( 'Documentation', 'webtoffee-product-feed' ) . '</a>',
		'<a target="_blank" href="https://wordpress.org/support/plugin/webtoffee-product-feed/reviews?rate=5#new-post">' . __( 'Review', 'webtoffee-product-feed' ) . '</a>',
	);
	if ( array_key_exists( 'deactivate', $links ) ) {
		$links[ 'deactivate' ] = str_replace( '<a', '<a class="productfeed-deactivate-link"', $links[ 'deactivate' ] );
	}
	return array_merge( $plugin_links, $links );
}

	
	
	
	
	/**
	 * Gets the available tabs.
	 *
	 * @since 2.0.0
	 *
	 * @return tabs[]
	 */
	public function get_tabs() {

		$tabs			 = [
			'connection-manager' => 'Manage Connection',
		];
		$is_connected	 = $this->is_connected();
		if ( $is_connected || !empty( ($_GET[ 'fb_access_token' ]) ) ) { // phpcs:ignore csrf ok.
			$tabs[ 'sync-products' ]	 = 'Sync Products';
			$tabs[ 'logs' ]			 = 'Logs';
		}
		return $tabs;
	}

	public function update_fb_access_token( $value ) {

		update_option( WT_Fb_Catalog_Manager_Settings::OPTION_ACCESS_TOKEN, $value );
	}

	public function update_fb_user_id( $value ) {

		update_option( WT_Fb_Catalog_Manager_Settings::OPTION_USER_ID, $value );
	}

	public function get_fb_user_id() {

		return get_option( WT_Fb_Catalog_Manager_Settings::OPTION_USER_ID );
	}

	public function update_fb_business_id( $value ) {

		update_option( WT_Fb_Catalog_Manager_Settings::OPTION_FB_BUSINESS_ID, $value );
	}

	public function update_fb_catalog_id( $value ) {

		update_option( WT_Fb_Catalog_Manager_Settings::OPTION_FB_CATALOG_ID, $value );
	}

	public function get_access_token() {

		$access_token = get_option( WT_Fb_Catalog_Manager_Settings::OPTION_ACCESS_TOKEN, '' );

		return $access_token;
	}

	public function get_fb_catalog_id() {

		return get_option( WT_Fb_Catalog_Manager_Settings::OPTION_FB_CATALOG_ID, '' );
	}

	public function is_connected() {

		return (bool) $this->get_access_token();
	}

	public function get_fb_catalog_details() {


		return $this->get_fb_catalog_id();
		/*
		  $catalog_url = "https://graph.facebook.com/" . WT_Fb_Catalog_Manager_Settings::FB_VERSION . "/" . $this->get_fb_catalog_id() . "?access_token=" . $this->get_access_token();

		  $catalog_response	 = wp_remote_get( $catalog_url );
		  $catalog_details	 = wp_remote_retrieve_body( $catalog_response );
		  $catalog_details	 = json_decode( $catalog_details );
		  return $catalog_details;
		 * 
		 */
	}

	/**
	 * Disconnects the integration using the Graph API.
	 *
	 * @internal
	 *
	 * @since 2.0.0
	 */
	public function handle_disconnect() {

		check_admin_referer( WT_Fb_Catalog_Manager_Settings::DISCONNECT_ACTION );

		if ( !current_user_can( 'manage_woocommerce' ) ) {
			wp_die( __( 'You do not have permission to uninstall Facebook Business Extension.', 'webtoffee-product-feed' ) );
		}

		$user_id		 = $this->get_fb_user_id();
		$access_token	 = $this->get_access_token();

		$permission_revoke_url = "https://graph.facebook.com/{$user_id}/permissions?access_token={$access_token}";

		$response = wp_remote_request( $permission_revoke_url, array(
			'method' => 'DELETE'
		)
		);

		$this->update_fb_access_token( '' );
		$this->update_fb_user_id( '' );
		$this->update_fb_business_id( '' );
		$this->update_fb_catalog_id( '' );



		wp_safe_redirect( $this->get_settings_url() );
		exit;
	}

	public function get_settings_url() {

		return admin_url( 'admin.php?page=webtoffee-product-feed' );
	}

	/**
	 * Renders the settings page.
	 *
	 * @since 2.0.0
	 */
	public function render() {

		$tabs = $this->get_tabs();

		$current_tab = '';
		if ( !empty( ($_GET[ 'fbtab' ]) ) ) { // phpcs:ignore csrf ok.
			$current_tab = sanitize_text_field($_GET[ 'fbtab' ]);
		}


		if ( !current_user_can( 'manage_woocommerce' ) ) {
			wp_die( __( 'You do not have permission to view', 'webtoffee-product-feed' ) );
		}


		if ( !$current_tab ) {
			$current_tab = current( array_keys( $tabs ) );
		}


		if ( !empty( ($_GET[ 'fb_access_token' ]) ) ) { // phpcs:ignore csrf ok.


			$this->update_fb_access_token( sanitize_text_field( trim( $_GET[ 'fb_access_token' ] ) ) );
			$this->update_fb_user_id( sanitize_text_field( trim( $_GET[ 'fb_user_id' ] ) ) );
			$this->update_fb_business_id( sanitize_text_field( trim( $_GET[ 'fb_business_id' ] ) ) );

			if ( !empty( ($_GET[ 'fb_catalog_id' ]) ) ) { // phpcs:ignore csrf ok.
				$catalogs_data = isset( $_GET[ 'fb_catalog_id' ] ) ? (array) $_GET[ 'fb_catalog_id' ] : array();

				$this->update_fb_catalog_id( $catalogs_data );
			}
		}

		$is_connected = $this->is_connected();
		?>

		<div class="woocommerce ">


			<nav class="nav-tab-wrapper woo-nav-tab-wrapper" style="margin:0px;">

				<?php foreach ( $tabs as $id => $label ) : ?>
					<a href="<?php echo esc_html( admin_url( 'admin.php?page=' . WT_Fb_Catalog_Manager_Settings::PAGE_ID . '&fbtab=' . esc_attr( $id ) ) ); ?>" class="nav-tab <?php echo $current_tab === $id ? 'nav-tab-active' : ''; ?>"><?php echo esc_html( $label ); ?></a>
				<?php endforeach; ?>

			</nav>
			<div class="wt-fbfeed-tab-container">
				<!--<h2><?php //_e( 'Get started with Facebook product synchronization', 'webtoffee-product-feed' );  ?></h2>-->


				<?php if ( 'connection-manager' === $current_tab ): ?>
					<h2><?php _e( 'Get started with Facebook product synchronization', 'webtoffee-product-feed' ); ?></h2>
					<div class="actions">
						<?php
						if ( !$is_connected ):
							?>
							<div class="wt-fbfeed-tab-content">

								<p><?php esc_html_e( 'You must connect with your FB business account as a pre-requisite to start synchronizing your products with Facebook.', 'webtoffee-product-feed' ); ?></p>
								<div class="not-connected-doc" style="background:#ebf3f7; color:#000; padding:10px;width:48%;margin-bottom:25px;">
									<p><?php esc_html_e( 'You will need to create a new', 'webtoffee-product-feed' ); ?> <a target="_blank" href="https://business.facebook.com/"> <b>Business Manager account</b> </a> <?php esc_html_e( 'to be able to connect your catalogue.', 'webtoffee-product-feed' ); ?> </p>
									<p><?php esc_html_e( 'Create your', 'webtoffee-product-feed' ); ?> <a target="_blank" href="https://www.facebook.com/business/help/1275400645914358"><b>product catalog</b></a> <?php esc_html_e( 'if you haven\'t one.', 'webtoffee-product-feed' ); ?> </p>
									<p>Use this <a target="_blank" href="https://developers.facebook.com/docs/marketing-api/catalog-batch/reference#supported-fields-items-batch"><b>reference</b></a> to view the product fields that will be synchronized via this plugin.</p>
								</div>


								<?php
								$actions = [
									'get-started' => [
										'label'	 => __( 'Connect with my FB product catalog', 'webtoffee-product-feed' ),
										'type'	 => 'primary',
										'url'	 => $this->get_connect_url(),
									],
								];
								?>


								<?php foreach ( $actions as $action_id => $action ) : ?>

									<a
										href="<?php echo esc_url( $action[ 'url' ] ); ?>"
										style="background:#1877f2 !important;" class="button button-<?php echo esc_attr( $action[ 'type' ] ); ?>"
										<?php echo ( 'get-started' !== $action_id ) ? 'target="_blank"' : ''; ?>
										>
											<?php echo esc_html( $action[ 'label' ] ); ?>
									</a>
									<p></p>

								<?php endforeach; ?>

							</div>

						<?php endif; ?>


						<?php
						if ( $is_connected ) :
							$catalog_details = $this->get_fb_catalog_details();
							?>


							<p><?php esc_html_e( 'You are currently connected to your FB account.', 'webtoffee-product-feed' ); ?></p>
							<?php if ( isset( $catalog_details ) ): ?>

								<div class="dashicons-before dashicons-cart">Catalogs associated with your FB account:
									<?php
									if ( !empty( $catalog_details ) ) {
										$ic = 0;
										foreach ( $catalog_details as $catalog_id => $catalog_name ):
											if ( $ic !== 0 )
												echo ', ';
											?>

									<b><a target="_blank" href="<?php echo "https://facebook.com/products/catalogs/" . $catalog_id; ?>"><?php echo esc_attr($catalog_name); ?></a></b>								
											<?php
											$ic++;
										endforeach;
									}
									?>
								</div>
							<?php else: ?>
								<p>Something went wrong with the connection establishment to catalogs, please refresh or try disconnecting and connect again to FB</p>
							<?php endif; ?>

							<br/>
							<p style="background:#ebf3f7; color:#000; padding:10px;width:42%;"><span>Use this <a target="_blank" href="https://developers.facebook.com/docs/marketing-api/catalog-batch/reference#supported-fields-items-batch"><b>reference</b></a> to view the product fields that will be synchronized via this plugin.</span></p>
							<br/>
							<a href="<?php echo esc_url( $this->get_disconnect_url() ); ?>" class="uninstall">
								<?php esc_html_e( 'Disconnect FB', 'webtoffee-product-feed' ); ?>
							</a>
							<p></p>
							<p></p>

						<?php endif; ?>
					</div>




				<?php elseif ( 'sync-products' === $current_tab ): ?>

					<?php
					if ( $is_connected ) :
						$wc_path			 = self::wt_get_wc_path();
						wp_enqueue_script( 'wc-enhanced-select' );
						wp_enqueue_style( 'woocommerce_admin_styles', $wc_path . '/assets/css/admin.css' );
						?>
						<p id="sync-loader" style="text-align:center">
							<i>Fetching the product categories, tags for sync with your FB Catalog ...</i>
							<span class="spinner is-active" style="float:none; margin-top: -3px;"></span>
						<p>
						<div class="sync-product-tab" style="display: none;">



							<div id="example-basic">

								<h3>Filter Products</h3>

								<section>
									<h2>Filter products for FB sync</h2>
									<h4><?php esc_html_e( 'Filter data necessary for sync with my FB Catalog as per the below criteria by excluding non-relevant products.', 'webtoffee-product-feed' ); ?></h4>
									<form action="" name="sync_products" id="sync_products" class="sync_products" method="post" autocomplete="off">	
										<?php wp_nonce_field( 'wt-sync-products' ); ?>
										<input type="hidden" name="wt_batch_hash_key" id="wt_batch_hash_key" value="<?php echo wp_generate_uuid4(); ?>"/>					
										<table class="form-table">
											<tr>
												<th><?php esc_html_e( 'Select FB Catalog', 'webtoffee-product-feed' ); ?></th>
												<td>
													<select name="wt_sync_selected_catalog"  >
														<?php
														$product_catalogs	 = $this->get_fb_catalog_details();
														foreach ( $product_catalogs as $catalog_id => $catalog_name ) {
															?>
														<option value="<?php echo $catalog_id; ?>" ><?php echo esc_attr($catalog_name); ?></option>								
															<?php
														}
														?>

													</select>
												</td>
											</tr>
											<tr>
												<th><?php esc_html_e( 'Exclude Product Categories', 'webtoffee-product-feed' ); ?></th>
												<td>
													<select name="wt_sync_exclude_category[]" class="wc-enhanced-select" multiple="multiple" data-placeholder ="<?php echo __( 'Search for a product category&hellip;', 'webtoffee-product-feed' ); ?>" >
														<?php
														$product_categories = $this->get_product_categories();
														foreach ( $product_categories as $category_id => $category_name ) {
															?>
														<option value="<?php echo $category_id; ?>" ><?php echo esc_attr($category_name); ?></option>								
															<?php
														}
														?>

													</select>
												</td>
											</tr>

											<tr>
												<th><?php esc_html_e( 'Exclude Product Tags', 'webtoffee-product-feed' ); ?></th>
												<td>
													<select name="wt_sync_exclude_tags[]" class="wc-enhanced-select" multiple="multiple" data-placeholder ="<?php echo __( 'Search for a product tag&hellip;', 'webtoffee-product-feed' ); ?>" >
														<?php
														$product_tags = $this->get_product_tags();
														foreach ( $product_tags as $product_tag_id => $product_tag_name ) {
															?>
															<option value="<?php echo $product_tag_id; ?>" ><?php echo esc_attr($product_tag_name); ?></option>								
															<?php
														}
														?>

													</select>
												</td>
											</tr>
											<tr>
												<th><?php esc_html_e( 'Products per batch', 'webtoffee-product-feed' ); ?></th>
												<td>

													<input type="text" name="wt_sync_batch_count" value="10" /><br/><br/>
													<i><?php esc_html_e( 'The number of records that the server will process for every iteration within the available server timeout interval. If the process fails you can lower this number accordingly and try again. Defaulted to 10 records. Maximum number allowed as per the Facebook limits is 5000.', 'webtoffee-product-feed' ); ?></i>
												</td>
											</tr>
				<!--									<tr>
												<td style="padding-left:0px;">
													<button name="syncproducts" id="syncproducts" type="submit" class="button button-large button-primary edd-export-form"><?php //esc_html_e( 'Sync Products', 'webtoffee-product-feed' ); ?></button>
												</td>
											</tr>-->
										</table>

									</form>

								</section>
								<h3>Map Categories</h3>
								<section id="category-section">													
									<?php
									$ajax_render = true;
									require plugin_dir_path( dirname( __FILE__ ) ) . 'includes/fbcatalog/partials/wt-fbfeed-category-mapping.php';
									?>
								</section>
								<!--												<h3>Pager</h3>
																				<section>
																					<p>The next and previous buttons help you to navigate through your content.</p>
																					<button name="syncproducts" id="syncproducts" type="submit" class="button button-large button-primary edd-export-form"><?php esc_html_e( 'Sync Products', 'webtoffee-product-feed' ); ?></button>
																				</section>-->
							</div>





							<style>
								.edd-progress div{width:0px;background:#1877f2;height:10px}
							</style>
						</div>

					<?php endif; ?>
					<?php else: ?>
					<div class="wt-fbfeed-tab-content">			
						<?php
						$batch_pocess_log = get_option( 'wt_feed_processed_handles', array() );

						if ( !empty( $batch_pocess_log ) ) {
							$this->list_batch_logs();
						}
						?>
					</div>
		<?php endif; ?>
			</div>
		</div>

		<?php
	}

	/**
	 * Get WC Plugin path without fail on any version
	 */
	public static function wt_get_wc_path() {
		if ( function_exists( 'WC' ) ) {
			$wc_path = WC()->plugin_url();
		} else {
			$wc_path = plugins_url() . '/woocommerce';
		}
		return $wc_path;
	}

	/**
	 * Gets the product categories.
	 * 
	 * @return array
	 */
	public function get_product_categories() {

		$term_query = new \WP_Term_Query( [
			'taxonomy'	 => 'product_cat',
			'hide_empty' => false,
			'fields'	 => 'id=>name',
		] );

		$product_categories = $term_query->get_terms();
		return is_array( $product_categories ) ? $product_categories : [];
	}

	/**
	 * Gets the product tags.
	 *
	 * @return array
	 */
	public function get_product_tags() {


		$term_query = new \WP_Term_Query( [
			'taxonomy'		 => 'product_tag',
			'hide_empty'	 => false,
			'hierarchical'	 => false,
			'fields'		 => 'id=>name',
		] );

		$product_tags = $term_query->get_terms();
		return is_array( $product_tags ) ? $product_tags : [];
	}

	/**
	 * Gets the URL for connecting.
	 * 
	 * @return string
	 */
	public function get_connect_url() {

		return add_query_arg( rawurlencode_deep( $this->get_connect_parameters() ), WT_Fb_Catalog_Manager_Settings::OAUTH_URL );
	}

	/**
	 * Gets the URL for disconnecting.
	 *
	 * @return string
	 */
	public function get_disconnect_url() {

		return wp_nonce_url( add_query_arg( 'action', WT_Fb_Catalog_Manager_Settings::DISCONNECT_ACTION, admin_url( 'admin.php' ) ), WT_Fb_Catalog_Manager_Settings::DISCONNECT_ACTION );
	}

	public function get_connect_parameters() {

		/**
		 * Filters the connection parameters.
		 *
		 * @param array $parameters connection parameters
		 */
		return apply_filters( 'wt_facebook_connection_parameters', [
			'client_id'		 => $this->get_client_id(),
			'redirect_uri'	 => "https://fbconnect.webtoffee.com/",
			'state'			 => admin_url( 'admin.php?page=webtoffee-product-feed' ), //?nonce=' ).wp_create_nonce( WT_Fb_Catalog_Manager_Settings::CONNECT_ACTION ), //$this->get_redirect_url(),
			'display'		 => 'page',
			'response_type'	 => 'code',
			'scope'			 => implode( ',', $this->get_scopes() ),
			'extras'		 => json_encode( $this->get_connect_parameters_extras() ),
		] );
	}

	public function wt_fbfeed_attribute_mapping() {


		if ( count( $_POST ) && isset( $_POST[ 'map_to_attr' ] ) ) {

			check_admin_referer( 'wt-attribute-mapping' );

			$mapping_option = 'wt_fbfeed_attribute_mapping';

			$mapping_data = array_map( 'absint', ($_POST[ 'map_to_attr' ]) );
			foreach ( $mapping_data as $local_attribute_id => $fb_attribute_id ) {
				if ( $fb_attribute_id )
					update_term_meta( $local_category_id, 'wt_fb_category', $fb_attribute_id );
			}

			// Delete product categories dropdown cache
			wp_cache_delete( 'wt_fbfeed_dropdown_product_attributes' );

			if ( update_option( $mapping_option, $mapping_data, false ) ) { // phpcs:ignore WordPress.PHP.DiscouragedPHPFunctions.serialize_serialize
				update_option( 'wt_mapping_message', esc_html__( 'Mapping Added Successfully', 'webtoffee-product-feed' ), false );
				wp_safe_redirect( admin_url( 'admin.php?page=wt-fbfeed-attribute-mapping&wt_mapping_message=success' ) );
				die();
			} else {
				update_option( 'wt_mapping_message', esc_html__( 'Failed To Add Mapping', 'webtoffee-product-feed' ), false );
				wp_safe_redirect( admin_url( 'admin.php?page=wt-fbfeed-attribute-mapping&wt_mapping_message=error' ) );
				die();
			}
		}
		require plugin_dir_path( dirname( __FILE__ ) ) . 'includes/fbcatalog/partials/wt-fbfeed-attribute-mapping.php';
	}

	private function get_client_id() {

		return WT_Fb_Catalog_Manager_Settings::CLIENT_ID;
	}

	public function get_batch_limit() {

		return apply_filters( "wt_fbfeed_upload_limit", $this->batch_limit, $this );
	}

	public function get_total_exported() {
		return ( ( $this->get_page() - 1 ) * $this->get_limit() ) + $this->exported_row_count;
	}

	public function get_percent_complete( $found_posts, $step, $limit ) {
		return $found_posts ? floor( ( ($step * $limit) / $found_posts ) * 100 ) : 100;
	}

	/**
	 * Gets the scopes that will be requested during the connection flow.
	 *
	 * @since 2.0.0
	 *
	 * @link https://developers.facebook.com/docs/marketing-api/access/#access_token
	 *
	 * @return string[]
	 */
	public function get_scopes() {

		$scopes = [
			'catalog_management',
			'business_management',
		//'ads_management',
		//'ads_read',
		];


		return (array) apply_filters( 'wt_facebook_connection_scopes', $scopes, $this );
	}

	private function get_connect_parameters_extras() {

		$parameters = [
			'setup'				 => [
				'external_business_id'	 => $this->get_external_business_id(),
				'timezone'				 => $this->get_timezone_string(),
				'currency'				 => get_woocommerce_currency(),
				'business_vertical'		 => 'ECOMMERCE',
			],
			'business_config'	 => [
				'business' => [
					'name' => $this->get_business_name(),
				],
			],
			'repeat'			 => false,
		];


		return $parameters;
	}

	public function get_external_business_id() {

		if ( !is_string( $this->external_business_id ) ) {

			$value = get_option( WT_Fb_Catalog_Manager_Settings::OPTION_EXTERNAL_BUSINESS_ID );

			if ( !is_string( $value ) ) {

				$value = uniqid( sanitize_title( $this->get_business_name() ) . '-', false );

				update_option( WT_Fb_Catalog_Manager_Settings::OPTION_EXTERNAL_BUSINESS_ID, $value );
			}

			$this->external_business_id = $value;
		}


		return (string) apply_filters( 'wt_facebook_external_business_id', $this->external_business_id, $this );
	}

	public function get_business_name() {

		$business_name = get_bloginfo( 'name' );

		$business_name = trim( (string) apply_filters( 'wt_facebook_connection_business_name', is_string( $business_name ) ? $business_name : '' ) );

		if ( empty( $business_name ) ) {
			$business_name = get_bloginfo( 'url' );
		}

		return html_entity_decode( $business_name, ENT_QUOTES, 'UTF-8' );
	}

	private function get_timezone_string() {

		$timezone = wc_timezone_string();

		if ( preg_match( '/([+-])(\d{2}):\d{2}/', $timezone, $matches ) ) {

			$hours		 = (int) $matches[ 2 ];
			$timezone	 = "Etc/GMT{$matches[ 1 ]}{$hours}";
		}

		return $timezone;
	}

	public function wt_fbfeed_category_mapping() {


		if ( count( $_POST ) && isset( $_POST[ 'map_to' ] ) ) {

			check_admin_referer( 'wt-category-mapping' );

			$mapping_option = 'wt_fbfeed_category_mapping';

			$mapping_data = array_map( 'absint', ($_POST[ 'map_to' ]) );
			foreach ( $mapping_data as $local_category_id => $fb_category_id ) {
				if ( $fb_category_id )
					update_term_meta( $local_category_id, 'wt_fb_category', $fb_category_id );
			}

			// Delete product categories dropdown cache
			wp_cache_delete( 'wt_fbfeed_dropdown_product_categories' );

			if ( update_option( $mapping_option, $mapping_data, false ) ) { // phpcs:ignore WordPress.PHP.DiscouragedPHPFunctions.serialize_serialize
				update_option( 'wt_mapping_message', esc_html__( 'Mapping Added Successfully', 'webtoffee-product-feed' ), false );
				wp_safe_redirect( admin_url( 'admin.php?page=wt-fbfeed-category-mapping&wt_mapping_message=success' ) );
				die();
			} else {
				update_option( 'wt_mapping_message', esc_html__( 'Failed To Add Mapping', 'webtoffee-product-feed' ), false );
				wp_safe_redirect( admin_url( 'admin.php?page=wt-fbfeed-category-mapping&wt_mapping_message=error' ) );
				die();
			}
		}
		require plugin_dir_path( dirname( __FILE__ ) ) . 'includes/fbcatalog/partials/wt-fbfeed-category-mapping.php';
	}

	/**
	 * Process batch exports via ajax
	 *
	 * @since 2.4
	 * @return void
	 */
	function wt_fbfeed_ajax_upload() {



		parse_str( $_POST[ 'form' ], $form );

		$_REQUEST	 = $form		 = (array) $form;
		check_admin_referer( 'wt-sync-products' );
		$step		 = absint( $_POST[ 'step' ] );

		$wt_batch_hash_key = sanitize_text_field( $_REQUEST[ 'wt_batch_hash_key' ] );

		$wt_sync_selected_catalog = sanitize_text_field( $_REQUEST[ 'wt_sync_selected_catalog' ] );


		$wt_sync_exclude_category	 = isset( $_REQUEST[ 'wt_sync_exclude_category' ] ) ? array_map( 'absint', ($_REQUEST[ 'wt_sync_exclude_category' ]) ) : [];
		$wt_sync_exclude_tags		 = isset( $_REQUEST[ 'wt_sync_exclude_tags' ] ) ? array_map( 'absint', ($_REQUEST[ 'wt_sync_exclude_tags' ]) ) : [];
		$wt_sync_batch_count		 = isset( $_REQUEST[ 'wt_sync_batch_count' ] ) ? absint( $_REQUEST[ 'wt_sync_batch_count' ] ) : $this->get_batch_limit();

		if ( $wt_sync_batch_count == 0 || $wt_sync_batch_count > 5000 ) {
			$wt_sync_batch_count = $this->get_batch_limit();
		}


		$product_data	 = [];
		$wc_fbfeed		 = new WT_Facebook_Catalog_Product();
		$args			 = array(
			'post_type'		 => array( 'product', 'product_variation' ),
			'post_status'	 => array( 'publish' ),
			'posts_per_page' => $wt_sync_batch_count,
			'offset'		 => ($step - 1) * $wt_sync_batch_count,
			'fields'		 => 'ids',
		/*
		  'tax_query'		 => array(
		  array(
		  'taxonomy'	 => 'product_type',
		  'field'		 => 'slug',
		  'terms'		 => array('simple', 'variable', 'variation'),
		  'operator'	 => 'IN'
		  ) )
		 * 
		 */
		);



		if ( !empty( $wt_sync_exclude_category ) ) {
			$args[ 'tax_query' ][] = array(
				'taxonomy'	 => 'product_cat',
				'terms'		 => $wt_sync_exclude_category, // Term ids to be excluded
				'operator'	 => 'NOT IN' // Excluding terms
			);
		}


		if ( !empty( $wt_sync_exclude_tags ) ) {
			$args[ 'tax_query' ][] = array(
				'taxonomy'	 => 'product_tag',
				'terms'		 => $wt_sync_exclude_tags, // Term ids to be excluded
				'operator'	 => 'NOT IN' // Excluding terms
			);
		}
		if ( !empty( $wt_sync_exclude_category ) || !empty( $wt_sync_exclude_tags ) ) {
			$args[ 'tax_query' ][ 'relation' ] = 'AND';
		}

		$loop = new WP_Query( $args );
		$process_products = apply_filters('wt_facebook_sync_products', $loop->posts);
		
		foreach ( $process_products as $product_id ) {
			$product_item_data = $wc_fbfeed->process_item_update( $product_id );

			if ( !empty( $product_item_data[ 'data' ][ 'price' ] ) ) {
				$product_data[] = $product_item_data;
			}
		}

		$catalog_access_token = $this->get_access_token();

		$request_body	 = [
			"headers"	 => [
				"Authorization"	 => "Bearer {$catalog_access_token}",
				"Content-type"	 => "application/json",
				"accept"		 => "application/json" ],
			"body"		 => json_encode( [
				"allow_upsert"	 => true,
				"item_type"		 => "PRODUCT_ITEM",
				"requests"		 => json_encode( $product_data )
			] ),
		];
		$catalog_id		 = $wt_sync_selected_catalog; //$this->get_fb_catalog_id();
		// Each bacth process the batch_limit		
		if ( !empty( $product_data ) ) {

			$this->wt_log_data_change( 'wt-feed-upload', 'Requested Product Data:' );
			$this->wt_log_data_change( 'wt-feed-upload', print_r( $product_data, 1 ) );

			//$catalog_id				 = $this->get_fb_catalog_id();
			#$batch_url				 = "https://graph.facebook.com/v13.0/$catalog_id/batch";
			$items_batch			 = "https://graph.facebook.com/v13.0/$catalog_id/items_batch";
			#$single_product_url	 = "https://graph.facebook.com/v13.0/$catalog_id/products";
			$batch_response			 = wp_remote_post( $items_batch, $request_body );
			$this->wt_log_data_change( 'wt-feed-upload', 'Batch Response:' );
			$this->wt_log_data_change( 'wt-feed-upload', print_r( $batch_response, 1 ) );
			$batch_response_details	 = wp_remote_retrieve_body( $batch_response );
			$batch_response_details	 = json_decode( $batch_response_details );

			$batch_pocess_log = get_option( 'wt_feed_processed_handles', array() );
			if ( isset( $batch_response_details->handles[ 0 ] ) ) {
				$batch_pocess_log[ $wt_batch_hash_key ][] = [
					'batch_time'	 => date( 'Y-m-d: H:i:s' ),
					'batch_handle'	 => $batch_response_details->handles[ 0 ],
					'catalog_id'	 => $catalog_id
				];
				update_option( 'wt_feed_processed_handles', $batch_pocess_log );
			}
		}
		//exit;

		$percentage_completed = $this->get_percent_complete( $loop->found_posts, $step, $wt_sync_batch_count );

		if ( $percentage_completed !== 100 ) {

			$step += 1;
			echo json_encode( array( 'step' => $step, 'percentage' => $percentage_completed, 'products' => $loop->found_posts, ) );
			exit;
		} else {


			echo json_encode( array( 'step' => 'done', 'percentage' => 100, 'url' => admin_url( 'admin.php?page=webtoffee-product-feed' ), 'catalog' => 'https://facebook.com/products/catalogs/' . $catalog_id . '/sets', 'url' => admin_url( 'admin.php?page=webtoffee-product-feed' ) ) );
			exit;
		}
	}

	public function wt_log_data_change( $content = 'wt-feed-upload', $data = '' ) {


		if ( WC()->version < '2.7.0' ) {
			$this->log = new WC_Logger();
		} else {
			$this->log = wc_get_logger();
		}

		if ( WC()->version < '2.7.0' ) {
			$this->log->add( $content, $data );
		} else {
			$context = array( 'source' => $content );
			$this->log->log( "debug", $data, $context );
		}
	}

	public function get_batch_status( $handle, $fb_catalog_id ) {

		$access_token = $this->get_access_token();

		$batch_status_handle_check_url	 = "https://graph.facebook.com/v13.0/$fb_catalog_id/check_batch_request_status?handle=$handle&access_token=$access_token&load_ids_of_invalid_requests=1";
		$batch_status_response			 = wp_remote_get( $batch_status_handle_check_url );
		$batch_status_response_details	 = wp_remote_retrieve_body( $batch_status_response );
		$batch_status_response_details	 = json_decode( $batch_status_response_details );

		$this->wt_log_data_change( 'wt-feed-upload', 'Batch Status Response:' );
		$this->wt_log_data_change( 'wt-feed-upload', print_r( $batch_status_response, 1 ) );

		return $batch_status_response_details;
	}

	public function list_batch_logs() {
		?>

		<div class="wt_fbfeed_history_page">
			<h2 class="wp-heading-inline"><?php _e( 'Logs' ); ?></h2>
			<p>
		<?php _e( 'Lists log of failed product syncs, mostly required for debugging purposes.' ); ?>
				<span><a target="_blank" href="<?php echo admin_url( 'admin.php?page=wc-status&tab=logs' ); ?>">Uploaded data logs</a> (<i>wt-feed-upload)</i>)</span>
			</p>

		<?php
		$batch_pocess_logs = get_option( 'wt_feed_processed_handles', array() );
		if ( is_array( $batch_pocess_logs ) && count( $batch_pocess_logs ) > 0 ) {
			?>
				<table class="wp-list-table widefat fixed striped history_list_tb log_list_tb">
					<thead>
						<tr>
							<th class="log_file_name_col"><?php _e( "Batch Started at" ); ?></th>
							<th><?php _e( "Action" ); ?></th>
						</tr>
					</thead>
			<?php
			$batch_pocess_logs = array_reverse( $batch_pocess_logs );
			foreach ( $batch_pocess_logs as $key => $batch_pocess_log ) :
				?>
						<tbody>
				<?php
				$batch_pocess_log = array_reverse( $batch_pocess_log );
				foreach ( $batch_pocess_log as $single_batch_log ) {
					if ( isset( $single_batch_log[ 'batch_handle' ] ) ) {
						?>
									<tr>
										<td class="log_file_name_col"><a class="wt_fbfeed_view_log_btn" data-log-file="<?php echo $single_batch_log[ 'batch_handle' ]; ?>"><?php echo $single_batch_log[ 'batch_time' ]; ?></a></td>
										<td>

											<a class="wt_fbfeed_view_log_btn" data-batch-handle="<?php echo $single_batch_log[ 'batch_handle' ]; ?>" data-batch-handle-catalog="<?php echo $single_batch_log[ 'catalog_id' ]; ?>"><?php _e( "View Status" ); ?></a>

										</td>
									</tr>
						<?php
					}
				}
				?>
						</tbody>
			<?php endforeach; ?>
				</table>
						<?php
					} else {
						?>
				<h4 class="wt_fbfeed_history_no_records"><?php _e( "No logs found." ); ?></h4>
					<?php
				}
				?>

			<div class="wt_fbfeed_view_log wt_fbfeed_popup">
				<div class="wt_fbfeed_popup_hd">
					<span style="line-height:40px;" class="dashicons dashicons-media-text"></span>
					<span class="wt_fbfeed_popup_hd_label"><?php _e( 'View log' ); ?></span>
					<div class="wt_fbfeed_popup_close">X</div>
				</div>
				<div class="wt_fbfeed_log_container">

				</div>
			</div>



		</div>

		<?php
	}

	public function wt_fbfeed_batch_status() {

		if ( !empty( $_POST[ 'batch_handle' ] ) ) {
			$nonce = (isset($_POST['_wpnonce']) ? sanitize_text_field($_POST['_wpnonce']) : '');
			if(!wp_verify_nonce($nonce, WEBTOFFEE_PRODUCT_FEED_ID)){
				return false;
			}
			$handle				 = sanitize_text_field( $_POST[ 'batch_handle' ] );
			$catalog_id			 = sanitize_text_field( $_POST[ 'catalog_id' ] );
			$batch_status_data	 = $this->get_batch_status( $handle, $catalog_id );

			if ( isset( $batch_status_data->data ) ) {
				wp_send_json( [ "errors" => $batch_status_data->data[ 0 ]->errors, "status" => $batch_status_data->data[ 0 ]->status, 'batch_response' => $batch_status_data->data[ 0 ], 'ids_of_invalid_requests' => $batch_status_data->data[ 0 ]->ids_of_invalid_requests ] );
			} else {
//				echo '<pre>$batch_status_data:-';
//				print_r( $batch_status_data );
//				echo '</pre>';
				wp_send_json( [ "errors" => 'The request could not be handled', "status" => 'failed' , 'batch_response' => ''] );			
			}

			exit;
		}
	}

	/**
	 * Process batch exports via ajax
	 *
	 * @since 2.4
	 * @return void
	 */
	function wt_fbfeed_ajax_save_category() {


		parse_str( $_POST[ 'form' ], $form );

		$_REQUEST	 = $form		 = (array) $form;
		check_admin_referer( 'wt-category-mapping' );


		$mapping_option = 'wt_fbfeed_category_mapping';

		$map_to_cats = !empty(($_REQUEST[ 'map_to' ])) ? ($_REQUEST[ 'map_to' ]) : array();
		$mapping_data = array_map( 'absint',  $map_to_cats);
		
		foreach ( $mapping_data as $local_category_id => $fb_category_id ) {
			if ( $fb_category_id )
				update_term_meta( $local_category_id, 'wt_fb_category', $fb_category_id );
		}

		// Delete product categories dropdown cache
		wp_cache_delete( 'wt_fbfeed_dropdown_product_categories' );

		if ( update_option( $mapping_option, $mapping_data, false ) ) { // phpcs:ignore WordPress.PHP.DiscouragedPHPFunctions.serialize_serialize
			update_option( 'wt_mapping_message', esc_html__( 'Mapping Added Successfully', 'webtoffee-product-feed' ), false );
			echo json_encode( array( 'step' => 'done', 'message' => 'success' ) );
		} else {
			update_option( 'wt_mapping_message', esc_html__( 'Failed To Add Mapping', 'webtoffee-product-feed' ), false );
			echo json_encode( array( 'step' => 'done', 'message' => 'failure' ) );
		}

		exit();
	}

}
